unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, ShlObj, RFilefind, ShellApi;

const
    SearchStr='&Search';
    ContinueStr='&Continue';
type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    EditPath: TEdit;
    ButtonPathsBrowse: TButton;
    ButtonAddPath: TButton;
    ButtonPathDelete: TButton;
    ButtonClearList: TButton;
    GroupBox3: TGroupBox;
    EditFile: TEdit;
    Button5: TButton;
    Button6: TButton;
    ButtonFileDelete: TButton;
    ButtonClearFiles: TButton;
    ButtonSearch: TSpeedButton;
    ButtonPause: TSpeedButton;
    ButtonStop: TSpeedButton;
    ListViewResults: TListView;
    ButtonOpen: TButton;
    ButtonNarrow: TButton;
    ButtonClearResults: TButton;
    ListBoxPaths: TListBox;
    ListBoxFiles: TListBox;
    CheckBoxCaseSensitive: TCheckBox;
    EditSearching: TEdit;
    EditFileName: TEdit;
    EditSearchText: TEdit;
    RFilefind1: TRFilefind;
    procedure ButtonSearchClick(Sender: TObject);
    procedure ButtonPauseClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure ButtonPathsBrowseClick(Sender: TObject);
    procedure ButtonAddPathClick(Sender: TObject);
    procedure RFilefind1Completed(Sender: TObject);
    procedure EditPathExit(Sender: TObject);
    procedure ButtonPathDeleteClick(Sender: TObject);
    procedure ButtonClearListClick(Sender: TObject);
    procedure ButtonClearFilesClick(Sender: TObject);
    procedure ButtonFileDeleteClick(Sender: TObject);
    procedure ButtonNarrowClick(Sender: TObject);
    procedure ButtonOpenClick(Sender: TObject);
    procedure ButtonClearResultsClick(Sender: TObject);
    procedure RFilefind1ChangeDirectory(Sender: TObject;
      directory: String);
    procedure ListViewResultsDblClick(Sender: TObject);
    procedure RFilefind1FileFound(Sender: TObject; directory: String;
      fileinfo: TSearchRec);
    procedure RFilefind1TextFound(Sender: TObject; directory: String;
      fileinfo: TSearchRec; offset: Integer;
      var processNextMatch: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    procedure StartSearch;
    { Private declarations }
  public
    function BrowseForFolder(Msg: String; fileSystemOnly: Boolean; var path: String): Boolean;
    function ExecuteFile(const FileName, Params, DefaultDir: string;
           ShowCmd: Integer): THandle;
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  buf: Array[0..32767] of Char;
implementation

{$R *.DFM}

procedure TMainForm.ButtonSearchClick(Sender: TObject);
begin
    ButtonSearch.Enabled:=FALSE;
    ButtonPause.Enabled:=TRUE;
    ButtonStop.Enabled:=TRUE;

    GroupBox1.Enabled:=FALSE;
    GroupBox2.Enabled:=FALSE;
    GroupBox3.Enabled:=FALSE;
    ButtonNarrow.Enabled:=FALSE;

    if (listBoxPaths.Items.Count=0) then ButtonAddPathClick(Sender);
    if (RFileFind1.State=ssPaused) then
    begin
        RFilefind1.Continue;
    end
    else
        StartSearch;
end;

procedure TMainForm.ButtonPauseClick(Sender: TObject);
begin
    ButtonSearch.Caption:=ContinueStr;
    ButtonSearch.Enabled:=TRUE;
    ButtonPause.Enabled:=FALSE;
    ButtonStop.Enabled:=TRUE;
    RFilefind1.Pause;
end;

procedure TMainForm.ButtonStopClick(Sender: TObject);
begin
    RFilefind1.Stop;

    ButtonSearch.Caption:=SearchStr;
    ButtonSearch.Enabled:=TRUE;
    ButtonPause.Enabled:=FALSE;
    ButtonStop.Enabled:=FALSE;

    GroupBox1.Enabled:=TRUE;
    GroupBox2.Enabled:=TRUE;
    GroupBox3.Enabled:=TRUE;
    ButtonNarrow.Enabled:=TRUE;

    EditSearching.Text:='';

end;

procedure TMainForm.ButtonPathsBrowseClick(Sender: TObject);
var
    folder: String;
begin
    BrowseForFolder('Choose your path',TRUE,folder);
    EditPath.TExt:=folder;
end;

function TMainFOrm.BrowseForFolder(Msg: String; fileSystemOnly: Boolean; var path: String): Boolean;
var
    BrowseInfo: TBrowseInfo;
    IdList: PItemIdList;
    titleBuf: Array [0..MAX_PATH] of Char;
    msgBuf:   Array [0..MAX_PATH] of Char;
begin
    Result:=FALSE;
    if (SHGetSpecialFolderLocation(Handle, CSIDL_DESKTOP, IdList)=NOERROR) then
    begin
         BrowseInfo.hWndOwner:=Handle;
         BrowseInfo.pidlRoot:=IdList;
         BrowseInfo.pszDisplayName:=titleBuf;
         BrowseInfo.lpszTitle:=strPCopy(@msgbuf,Msg);
         BrowseInfo.ulFlags:=0;
         if (FileSystemOnly) then BrowseInfo.ulFlags:=BIF_RETURNONLYFSDIRS;
         BrowseInfo.lpfn:=nil;
         BrowseInfo.lparam:= 0;
         IDList := SHBrowseForFolder(BrowseInfo);
         if (assigned(IdList)) then
         begin
             if SHGetPathFromIdList(idList, @titleBuf)=True then
             begin
                 Result:=TRUE;
                 Path:=titleBuf;
             end;
         end;
    end;
end;


procedure TMainForm.ButtonAddPathClick(Sender: TObject);
begin

    if (listBoxPaths.Items.IndexOf(editPath.Text)=-1)and
       (length(editPath.Text)>=1) then
        listBoxPaths.Items.Add(editPath.Text);
end;

procedure TMainForm.StartSearch;
begin
    RFileFind1.ContainingText:=EditSearchText.Text;
    RFileFind1.FileMask:=EditFileName.Text;
    RFileFind1.Directories:=listboxPaths.Items;
    RFileFind1.Files:=listBoxFiles.Items;
    RFileFind1.CaseSensitive:=CheckBoxCaseSensitive.Checked;
    ListviewResults.Items.Clear;
    RFileFind1.Start;
end;


procedure TMainForm.RFilefind1Completed(Sender: TObject);
begin
    ButtonSearch.Caption:=SearchStr;
    ButtonStopClick(Sender);
end;

procedure TMainForm.EditPathExit(Sender: TObject);
begin
    if (listBoxPaths.Items.Count=0) then ButtonAddPathClick(Sender);
end;

procedure TMainForm.ButtonPathDeleteClick(Sender: TObject);
var
    r: Integer;
    Label Again;
begin
    Again:
    for r:=0 to listBoxPaths.Items.Count-1 do
        if (listBoxPaths.Selected[r]) then
        begin
            listBoxPaths.Items.Delete(r);
            goto Again;
        end;
end;


function TMainForm.ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;



procedure TMainForm.ButtonClearListClick(Sender: TObject);
begin
    ListBoxPaths.Items.Clear;
end;

procedure TMainForm.ButtonClearFilesClick(Sender: TObject);
begin
    ListBoxFiles.Items.Clear;
end;

procedure TMainForm.ButtonFileDeleteClick(Sender: TObject);
var
    r: Integer;
    Label Again;
begin
    Again:
    for r:=0 to listBoxPaths.Items.Count-1 do
        if (listBoxPaths.Selected[r]) then
        begin
            listBoxPaths.Items.Delete(r);
            goto Again;
        end;
end;


procedure TMainForm.ButtonNarrowClick(Sender: TObject);
var
    r: Integer;
begin
    ListBoxFiles.Items.Clear;
    for r:=0 to listviewResults.Items.Count-1 do
    begin
        ListBoxFiles.Items.Add(listviewResults.Items[r].Caption);
    end;
end;

procedure TMainForm.ButtonOpenClick(Sender: TObject);
begin
    if (listviewResults.Selected<>nil) then
        ExecuteFile(listviewResults.Selected.Caption,'','',SW_SHOWNORMAL);
end;

procedure TMainForm.ButtonClearResultsClick(Sender: TObject);
begin
    listviewResults.Items.Clear;
end;


procedure TMainForm.RFilefind1FileFound(Sender: TObject; directory: String;
  fileinfo: TSearchRec);
var
    item: TListItem;
    attr: String;
    sysTime: TSystemTime;
begin
    item:=listviewResults.Items.Add;
    item.Caption:=directory+fileinfo.Name;

    FileTimeToSystemTime(fileinfo.FindData.ftLastWriteTime,sysTime);

    // Delphi2 users: comment out next line
    item.SubItems.Add(DateTimeToStr(SystemTimeToDateTime(sysTime)));

    attr:='      ';
    if (fileinfo.Attr and $20<>0) then attr[1]:='A';
    if (fileinfo.Attr and $10<>0) then
    begin
        item.SubItems.Add('-');
        attr[2]:='D';
    end
    else item.SubItems.Add(intToStr(fileinfo.Size));
    if (fileinfo.Attr and $08<>0) then attr[3]:='V';
    if (fileinfo.Attr and $04<>0) then attr[4]:='S';
    if (fileinfo.Attr and $02<>0) then attr[5]:='H';
    if (fileinfo.Attr and $01<>0) then attr[6]:='R';

    item.SubItems.Add(attr);
end;

procedure TMainForm.RFilefind1ChangeDirectory(Sender: TObject;
  directory: String);
begin
    EditSearching.Text:='Searching: '+directory;
end;

procedure TMainForm.ListViewResultsDblClick(Sender: TObject);
begin
    ButtonOpenClick(Sender);
end;


procedure TMainForm.RFilefind1TextFound(Sender: TObject; directory: String;
  fileinfo: TSearchRec; offset: Integer; var processNextMatch: Boolean);
begin
    RFileFind1FileFound(Sender,directory,fileinfo);
    processNextMatch:=FALSE;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
    ClientWidth:=640;
    ClientHeight:=480;
end;

end.
